@@ -29,6 +29,7 @@ from calendarweek.django import CalendarWeek, i18n_day_abbr_choices_lazy, i18n_d
 from celery.result import allow_join_result
 from celery.states import SUCCESS
 from colorfield.fields import ColorField
+from model_utils import FieldTracker
 from reversion.models import Revision, Version
 
 from aleksis.apps.chronos.managers import (
@@ -427,6 +428,8 @@ class Lesson(ValidityRangeRelatedExtensibleModel, GroupPropertiesMixin, TeacherP
 class LessonSubstitution(ExtensibleModel, TeacherPropertiesMixin, WeekRelatedMixin):
     objects = LessonSubstitutionManager.from_queryset(LessonSubstitutionQuerySet)()
 
+    tracker = FieldTracker()
+
     week = models.IntegerField(verbose_name=_("Week"), default=CalendarWeek.current_week)
     year = models.IntegerField(verbose_name=_("Year"), default=get_current_year)
 
@@ -466,6 +469,16 @@ class LessonSubstitution(ExtensibleModel, TeacherPropertiesMixin, WeekRelatedMix
         week = CalendarWeek(week=self.week, year=self.year)
         return week[self.lesson_period.period.weekday]
 
+    @property
+    def time_range(self) -> (timezone.datetime, timezone.datetime):
+        """Get the time range of this substitution."""
+        return timezone.datetime.combine(
+            self.date, self.lesson_period.period.time_start
+        ), timezone.datetime.combine(self.date, self.lesson_period.period.time_end)
+
+    def get_teachers(self):
+        return self.teachers
+
     def __str__(self):
         return f"{self.lesson_period}, {date_format(self.date)}"
 
@@ -998,6 +1011,8 @@ class Supervision(ValidityRangeRelatedExtensibleModel, WeekAnnotationMixin):
 class SupervisionSubstitution(ExtensibleModel):
     objects = SupervisionSubstitutionManager()
 
+    tracker = FieldTracker()
+
     date = models.DateField(verbose_name=_("Date"))
     supervision = models.ForeignKey(
         Supervision,
@@ -1016,6 +1031,17 @@ class SupervisionSubstitution(ExtensibleModel):
     def teachers(self):
         return [self.teacher]
 
+    @property
+    def time_range(self) -> (timezone.datetime, timezone.datetime):
+        """Get the time range of this supervision substitution."""
+        return timezone.datetime.combine(
+            self.date,
+            self.supervision.break_item.time_start or self.supervision.break_item.time_end,
+        ), timezone.datetime.combine(
+            self.date,
+            self.supervision.break_item.time_end or self.supervision.break_item.time_start,
+        )
+
     def __str__(self):
         return f"{self.supervision}, {date_format(self.date)}"
 
@@ -1028,6 +1054,8 @@ class SupervisionSubstitution(ExtensibleModel):
 class Event(SchoolTermRelatedExtensibleModel, GroupPropertiesMixin, TeacherPropertiesMixin):
     label_ = "event"
 
+    tracker = FieldTracker()
+
     objects = EventManager.from_queryset(EventQuerySet)()
 
     title = models.CharField(verbose_name=_("Title"), max_length=255, blank=True, null=True)
@@ -1126,6 +1154,13 @@ class Event(SchoolTermRelatedExtensibleModel, GroupPropertiesMixin, TeacherPrope
         """Get teachers relation."""
         return self.teachers
 
+    @property
+    def time_range(self) -> (timezone.datetime, timezone.datetime):
+        """Get the time range of this event."""
+        return timezone.datetime.combine(
+            self.date_start, self.period_from.time_start
+        ), timezone.datetime.combine(self.date_end, self.period_to.time_end)
+
     class Meta:
         # Heads up: Link to period implies uniqueness per site
         ordering = ["date_start"]
@@ -1145,6 +1180,8 @@ class ExtraLesson(
 ):
     label_ = "extra_lesson"
 
+    tracker = FieldTracker()
+
     objects = ExtraLessonManager.from_queryset(ExtraLessonQuerySet)()
 
     week = models.IntegerField(verbose_name=_("Week"), default=CalendarWeek.current_week)
@@ -1195,6 +1232,13 @@ class ExtraLesson(
         """Get subject."""
         return self.subject
 
+    @property
+    def time_range(self) -> (timezone.datetime, timezone.datetime):
+        """Get the time range of this extra lesson."""
+        return timezone.datetime.combine(
+            self.date, self.period.time_start
+        ), timezone.datetime.combine(self.date, self.period.time_end)
+
     class Meta:
         # Heads up: Link to period implies uniqueness per site
         verbose_name = _("Extra lesson")
